@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2022 Pier Carlo Chiodi
+# Copyright (C) 2017-2023 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -291,7 +291,7 @@ class ConfigureCommand(ARouteServerCommand):
         filtering["ipv4_pref_len"] = {"min": 8, "max": 24}
         filtering["ipv6_pref_len"] = {"min": 12, "max": 48}
         self.notes.append(
-            "Accepted prefix lengths are 8-24 for IPv6 and 12-48 for IPv6."
+            "Accepted prefix lengths are 8-24 for IPv4 and 12-48 for IPv6."
         )
 
         filtering["global_black_list_pref"] = []
@@ -343,16 +343,9 @@ class ConfigureCommand(ARouteServerCommand):
             "RPKI ROAs are used as if they were route objects to further "
             "enrich IRR data."
         )
-        irrdb["use_arin_bulk_whois_data"] = {
-            "enabled": True
-        }
         irrdb["use_registrobr_bulk_whois_data"] = {
             "enabled": True
         }
-        self.notes.append(
-            "ARIN Whois database dump is fetched from NLNOG to further "
-            "enrich IRR data."
-        )
         self.notes.append(
             "NIC.BR Whois database dump is fetched from Registro.br to further "
             "enrich IRR data."
@@ -376,6 +369,22 @@ class ConfigureCommand(ARouteServerCommand):
             "PeeringDB is used to fetch networks prefix count."
         )
 
+        if (
+            (
+                self.answers["daemon"] == "bird" and \
+                version.parse(self.answers["version"]) >= version.parse("2.0.11")
+            ) or (
+                self.answers["daemon"] == "openbgpd" and \
+                version.parse(self.answers["version"]) >= version.parse("7.8")
+            )
+        ):
+            filtering["roles"] = {
+                "enabled": True
+            }
+            self.notes.append(
+                "Route leak prevention using roles (RFC9234) is enabled."
+            )
+
         if self.answers["daemon"] == "bird":
             filtering["reject_policy"] = {
                 "policy": "tag_and_reject"
