@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2022 Pier Carlo Chiodi
+# Copyright (C) 2017-2023 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@ class ConfigParserBase(object):
         def expand_env_vars(doc):
             res = doc
             res = os.path.expandvars(res)
-            res = re.sub("\$\{[A-Za-z0-9_]+\}", "", res)
+            res = re.sub(r"\$\{[A-Za-z0-9_]+\}", "", res)
             return res
 
         lines = doc.split("\n")
