@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2022 Pier Carlo Chiodi
+# Copyright (C) 2017-2023 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ class RTTGetter_WorkerThread(BaseConfigEnricherThread):
     DESCR = "RTTGetter"
 
     # The following regex pattern is reported by docs/RTT_GETTER.rst
-    RETURN_VALUE_RE_PATTERN = re.compile("^\d+[.]?\d*$")
+    RETURN_VALUE_RE_PATTERN = re.compile(r"^\d+[.]?\d*$")
 
     def __init__(self, *args, **kwargs):
         BaseConfigEnricherThread.__init__(self, *args, **kwargs)
