@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2022 Pier Carlo Chiodi
+# Copyright (C) 2017-2023 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -186,8 +186,8 @@ class EuroIXMemberList(object):
         def normalize_bgp_community(s):
             res = s.decode("utf-8")
             res = res.lower()
-            res = re.sub("[\\/\[ \]+-]", "_", res)
-            res = re.sub("[^0-9a-zA-Z_]", "", res)
+            res = re.sub(r"[\\/\[ \]+-]", "_", res)
+            res = re.sub(r"[^0-9a-zA-Z_]", "", res)
             return res
 
         def attach_custom_bgp_community(client, prefix, name):
