@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2022 Pier Carlo Chiodi
+# Copyright (C) 2017-2023 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -156,7 +156,7 @@ class OpenBGPDInstance(object):
         return None
 
     def _parse_routes(self, out):
-        regex = "BGP routing table entry for ([0-9\.\:a-f]+/[0-9]+)"
+        regex = r"BGP routing table entry for ([0-9\.\:a-f]+/[0-9]+)"
 
         route = {}
         routes = []
@@ -216,6 +216,8 @@ class OpenBGPDInstance(object):
                 route["ext_comms"] = line.split(": ")[1]
             elif line.startswith("Large Communities:"):
                 route["lrg_comms"] = line.split(": ")[1]
+            elif line.startswith("OTC:"):
+                route["otc"] = line.split(": ")[1]
             last_line_new_route = False
 
         if last_line_new_route:
@@ -238,7 +240,17 @@ class OpenBGPDInstance(object):
         out = self.run_cmd(
             "bgpctl -n show ip bgp detail"
         )
-        return self._parse_routes(out)
+        routes = self._parse_routes(out)
+
+        # Add rejected routes.
+        out = self.run_cmd(
+            "bgpctl -n show ip bgp detail error"
+        )
+        for rejected_route in self._parse_routes(out):
+            rejected_route.filtered = True
+            routes.append(rejected_route)
+
+        return routes
 
     def _get_routes_from_all_sources(self):
         self.routes = {
@@ -607,5 +619,25 @@ class OpenBGPD75PortableInstance(OpenBGPDPortableInstance):
     TARGET_VERSION = "7.5"
 
 
-OpenBGPDPortablePreviousInstance = OpenBGPD74PortableInstance
-OpenBGPDPortableLatestInstance = OpenBGPD75PortableInstance
+class OpenBGPD76PortableInstance(OpenBGPDPortableInstance):
+
+    DOCKER_IMAGE = "pierky/openbgpd:7.6"
+
+    TAG = "openbgpd76p"
+
+    BGP_SPEAKER_VERSION = "7.6"
+    TARGET_VERSION = "7.6"
+
+
+class OpenBGPD77PortableInstance(OpenBGPDPortableInstance):
+
+    DOCKER_IMAGE = "pierky/openbgpd:7.7"
+
+    TAG = "openbgpd77p"
+
+    BGP_SPEAKER_VERSION = "7.7"
+    TARGET_VERSION = "7.7"
+
+
+OpenBGPDPortablePreviousInstance = OpenBGPD76PortableInstance
+OpenBGPDPortableLatestInstance = OpenBGPD77PortableInstance
